@@ -6,7 +6,8 @@ use bevy_asset_loader::{AssetCollection, AssetLoader};
 /// Requires the feature 'render' (part of default features)
 fn main() {
     let mut app = App::build();
-    AssetLoader::new(MyStates::AssetLoading, MyStates::Next)
+    AssetLoader::new(MyStates::AssetLoading)
+        .continue_to_state(MyStates::Next)
         .with_collection::<MyAssets>()
         .build(&mut app);
     app.add_state(MyStates::AssetLoading)
@@ -22,7 +23,7 @@ fn main() {
 struct MyAssets {
     // if the sheet would have padding, we could set that with `padding_x` and `padding_y`
     #[asset(texture_atlas(tile_size_x = 100., tile_size_y = 96., columns = 8, rows = 1))]
-    #[asset(path = "textures/female_adventurer.png")]
+    #[asset(path = "textures/female_adventurer_sheet.png")]
     female_adventurer: Handle<TextureAtlas>,
 }
 
