@@ -3,10 +3,11 @@ use bevy_asset_loader::{AssetCollection, AssetLoader};
 
 /// This example demonstrates how to load a color material from a .png file
 ///
-/// Requires the feature 'sprite' (part of default features)
+/// Requires the feature 'render' (part of default features)
 fn main() {
     let mut app = App::build();
-    AssetLoader::new(MyStates::AssetLoading, MyStates::Next)
+    AssetLoader::new(MyStates::AssetLoading)
+        .continue_to_state(MyStates::Next)
         .with_collection::<MyAssets>()
         .build(&mut app);
     app.add_state(MyStates::AssetLoading)
