@@ -1,22 +1,25 @@
-//! The goal of this crate is to offer an easy way for bevy games to load all their assets in a loading State.
+//! The goal of this crate is to offer an easy way for bevy games to load all their assets in a loading [bevy_ecs::schedule::State].
 //!
 //! `bevy_asset_loader` introduces the derivable trait [AssetCollection]. Structs with asset handles
-//! can be automatically loaded during a configurable loading [State]. Afterwards they will be inserted as
-//! resources containing loaded handles and the plugin will switch to a second configurable [State].
+//! can be automatically loaded during a configurable loading [bevy_ecs::schedule::State]. Afterwards they will be inserted as
+//! resources containing loaded handles and the plugin will switch to a second configurable [bevy_ecs::schedule::State].
 //!
-//! ```edition2018
+//! ```edition2021
 //! # use bevy_asset_loader::{AssetLoader, AssetCollection};
 //! # use bevy::prelude::*;
 //! # use bevy::asset::AssetPlugin;
 //! fn main() {
-//! let mut app = App::build();
-//!     AssetLoader::new(GameState::Loading, GameState::Next)
+//!     let mut app = App::build();
+//!     AssetLoader::new(GameState::Loading)
+//!         .continue_to_state(GameState::Next)
 //!         .with_collection::<AudioAssets>()
 //!         .with_collection::<TextureAssets>()
 //!         .build(&mut app);
 //!     app
 //!         .add_state(GameState::Loading)
-//!         //.add_plugins(DefaultPlugins)
+//! # /*
+//!         .add_plugins(DefaultPlugins)
+//! # */
 //!         .add_system_set(SystemSet::on_update(GameState::Next)
 //!             .with_system(use_asset_handles.system())
 //!         )
@@ -42,8 +45,8 @@
 //!     pub tree: Handle<Texture>,
 //! }
 //!
-//! // since this function runs in [MyState::Next], we know our assets are
-//! // loaded and [MyAudioAssets] is a resource
+//! // since this function runs in MyState::Next, we know our assets are
+//! // loaded and their handles are in the resource AudioAssets
 //! fn use_asset_handles(audio_assets: Res<AudioAssets>, audio: Res<Audio>) {
 //!     audio.play(audio_assets.background.clone());
 //! }
@@ -75,7 +78,7 @@ use std::marker::PhantomData;
 ///
 /// Derive is supported for structs with named fields.
 /// Each field needs to be annotated with ``#[asset(path = "path/to/asset.file")]``
-/// ```edition2018
+/// ```edition2021
 /// # use bevy_asset_loader::AssetCollection;
 /// # use bevy::prelude::*;
 /// #[derive(AssetCollection)]
@@ -87,7 +90,7 @@ use std::marker::PhantomData;
 /// }
 /// ```
 pub trait AssetCollection: Component {
-    /// Create a new AssetCollection from the [bevy::asset::AssetServer]
+    /// Create a new AssetCollection from the [bevy_asset::AssetServer]
     fn create(world: &mut World) -> Self;
     /// Start loading all the assets in the collection
     fn load(world: &mut World) -> Vec<HandleUntyped>;
@@ -111,10 +114,102 @@ impl<T> Default for AssetLoaderConfiguration<T> {
 }
 
 struct LoadingConfiguration<T> {
-    next: T,
+    next: Option<T>,
     count: usize,
 }
 
+/// Resource to dynamically resolve keys to asset paths.
+///
+/// This resource is set by the [AssetLoader] and is read when entering a loading state.
+/// You should set your desired asset key and paths in a previous [bevy_ecs::schedule::State].
+///
+/// ```edition2021
+/// # use bevy::prelude::*;
+/// # use bevy_asset_loader::{AssetKeys, AssetCollection};
+/// fn choose_character(
+///     mut state: ResMut<State<GameState>>,
+///     mut asset_keys: ResMut<AssetKeys>,
+///     mouse_input: Res<Input<MouseButton>>,
+/// ) {
+///     if mouse_input.just_pressed(MouseButton::Left) {
+///         asset_keys.set_asset_key("character", "textures/female_adventurer.png")
+///     } else if mouse_input.just_pressed(MouseButton::Right) {
+///         asset_keys.set_asset_key("character", "textures/zombie.png")
+///     } else {
+///         return;
+///     }
+///
+///     state
+///         .set(GameState::Loading)
+///         .expect("Failed to change state");
+/// }
+///
+/// #[derive(AssetCollection)]
+/// struct TextureAssets {
+///     #[asset(key = "character")]
+///     player: Handle<Texture>,
+/// }
+/// # #[derive(Clone, Eq, PartialEq, Debug, Hash)]
+/// # enum GameState {
+/// #     Loading,
+/// #     Menu
+/// # }
+/// ```
+#[derive(Default)]
+pub struct AssetKeys {
+    keys: HashMap<String, String>,
+}
+
+impl AssetKeys {
+    /// Get the asset path corresponding to the given key.
+    ///
+    /// PANIC: if the key does not exist
+    pub fn get_path_for_key(&self, key: &str) -> &str {
+        self.keys
+            .get(key)
+            .expect(&format!("Failed to get a path for key '{}'", key))
+    }
+
+    /// Set the corresponding asset path for the given key.
+    ///
+    /// In case the key is already known, its value will be overwritten.
+    /// ```edition2021
+    /// # use bevy::prelude::*;
+    /// # use bevy_asset_loader::{AssetKeys, AssetCollection};
+    /// fn choose_character(
+    ///     mut state: ResMut<State<GameState>>,
+    ///     mut asset_keys: ResMut<AssetKeys>,
+    ///     mouse_input: Res<Input<MouseButton>>,
+    /// ) {
+    ///     if mouse_input.just_pressed(MouseButton::Left) {
+    ///         asset_keys.set_asset_key("character", "textures/female_adventurer.png")
+    ///     } else if mouse_input.just_pressed(MouseButton::Right) {
+    ///         asset_keys.set_asset_key("character", "textures/zombie.png")
+    ///     } else {
+    ///         return;
+    ///     }
+    ///
+    ///     state
+    ///         .set(GameState::Loading)
+    ///         .expect("Failed to change state");
+    /// }
+    ///
+    /// #[derive(AssetCollection)]
+    /// struct TextureAssets {
+    ///     #[asset(key = "character")]
+    ///     player: Handle<Texture>,
+    /// }
+    /// # #[derive(Clone, Eq, PartialEq, Debug, Hash)]
+    /// # enum GameState {
+    /// #     Loading,
+    /// #     Menu
+    /// # }
+    /// ```
+    pub fn set_asset_key<T: Into<String>>(&mut self, key: T, value: T) {
+        self.keys.insert(key.into(), value.into());
+    }
+}
+
 fn start_loading<T: Component + Debug + Clone + Eq + Hash, Assets: AssetCollection>(
     world: &mut World,
 ) {
@@ -163,11 +258,6 @@ fn check_loading_state<T: Component + Debug + Clone + Eq + Hash, Assets: AssetCo
             return;
         }
 
-        // Todo: fire events `AssetCollection-` Ready/Loaded/Inserted?
-        // First event when all handles are done
-        // => system checks for events, reduces config count/changes state
-        // => fires event that collection is now inserted
-        // Export labels to sort check_loading_state / insert systems
         let mut state = cell
             .get_resource_mut::<State<T>>()
             .expect("Cannot get State resource");
@@ -180,9 +270,9 @@ fn check_loading_state<T: Component + Debug + Clone + Eq + Hash, Assets: AssetCo
         {
             config.count -= 1;
             if config.count == 0 {
-                state
-                    .set(config.next.clone())
-                    .expect("Failed to set next State");
+                if let Some(next) = config.next.as_ref() {
+                    state.set(next.clone()).expect("Failed to set next State");
+                }
             }
         }
     }
@@ -198,13 +288,14 @@ fn init_resource<Asset: FromWorld + Component>(world: &mut World) {
 
 /// A Bevy plugin to configure automatic asset loading
 ///
-/// ```edition2018
+/// ```edition2021
 /// # use bevy_asset_loader::{AssetLoader, AssetCollection};
 /// # use bevy::prelude::*;
 /// # use bevy::asset::AssetPlugin;
 /// fn main() {
-/// let mut app = App::build();
-///     AssetLoader::new(GameState::Loading, GameState::Menu)
+///     let mut app = App::build();
+///     AssetLoader::new(GameState::Loading)
+///         .continue_to_state(GameState::Menu)
 ///         .with_collection::<AudioAssets>()
 ///         .with_collection::<TextureAssets>()
 ///         .build(&mut app);
@@ -244,8 +335,9 @@ fn init_resource<Asset: FromWorld + Component>(world: &mut World) {
 /// }
 /// ```
 pub struct AssetLoader<T> {
-    next_state: T,
+    next_state: Option<T>,
     loading_state: T,
+    keys: HashMap<String, String>,
     load: SystemSet,
     check: SystemSet,
     post_process: SystemSet,
@@ -258,15 +350,16 @@ where
 {
     /// Create a new [AssetLoader]
     ///
-    /// This function takes two States. During the first all assets will be loaded and the
-    /// collections will be inserted as resources. Then the second state is set in your Bevy App.
-    /// ```edition2018
+    /// This function takes a [bevy_ecs::schedule::State] during which all asset collections will
+    /// be loaded and inserted as resources.
+    /// ```edition2021
     /// # use bevy_asset_loader::{AssetLoader, AssetCollection};
     /// # use bevy::prelude::*;
     /// # use bevy::asset::AssetPlugin;
     /// # fn main() {
     ///     let mut app = App::build();
-    ///     AssetLoader::new(GameState::Loading, GameState::Menu)
+    ///     AssetLoader::new(GameState::Loading)
+    ///         .continue_to_state(GameState::Menu)
     ///         .with_collection::<AudioAssets>()
     ///         .with_collection::<TextureAssets>()
     ///         .build(&mut app);
@@ -295,10 +388,11 @@ where
     /// #     pub tree: Handle<Texture>,
     /// # }
     /// ```
-    pub fn new(load: State, next: State) -> AssetLoader<State> {
+    pub fn new(load: State) -> AssetLoader<State> {
         Self {
-            next_state: next,
+            next_state: None,
             loading_state: load.clone(),
+            keys: HashMap::default(),
             load: SystemSet::on_enter(load.clone()),
             check: SystemSet::on_update(load.clone()),
             post_process: SystemSet::on_exit(load),
@@ -306,16 +400,61 @@ where
         }
     }
 
-    /// Add an [AssetCollection] to the [AssetLoader]
-    ///
-    /// The added collection will be loaded and inserted into your Bevy App as a resource.
-    /// ```edition2018
+    /// The [AssetLoader] will set this [bevy_ecs::schedule::State] after all asset collections
+    /// are loaded and inserted as resources.
+    /// ```edition2021
     /// # use bevy_asset_loader::{AssetLoader, AssetCollection};
     /// # use bevy::prelude::*;
     /// # use bevy::asset::AssetPlugin;
     /// # fn main() {
     ///     let mut app = App::build();
-    ///     AssetLoader::new(GameState::Loading, GameState::Menu)
+    ///     AssetLoader::new(GameState::Loading)
+    ///         .continue_to_state(GameState::Menu)
+    ///         .with_collection::<AudioAssets>()
+    ///         .with_collection::<TextureAssets>()
+    ///         .build(&mut app);
+    /// #   app
+    /// #       .add_state(GameState::Loading)
+    /// #       .add_plugins(MinimalPlugins)
+    /// #       .add_plugin(AssetPlugin::default())
+    /// #       .set_runner(|mut app| app.schedule.run(&mut app.world))
+    /// #       .run();
+    /// # }
+    /// # #[derive(Clone, Eq, PartialEq, Debug, Hash)]
+    /// # enum GameState {
+    /// #     Loading,
+    /// #     Menu
+    /// # }
+    /// # #[derive(AssetCollection)]
+    /// # pub struct AudioAssets {
+    /// #     #[asset(path = "audio/background.ogg")]
+    /// #     pub background: Handle<AudioSource>,
+    /// # }
+    /// # #[derive(AssetCollection)]
+    /// # pub struct TextureAssets {
+    /// #     #[asset(path = "textures/player.png")]
+    /// #     pub player: Handle<Texture>,
+    /// #     #[asset(path = "textures/tree.png")]
+    /// #     pub tree: Handle<Texture>,
+    /// # }
+    /// ```
+    pub fn continue_to_state(mut self, next: State) -> Self {
+        self.next_state = Some(next);
+
+        self
+    }
+
+    /// Add an [AssetCollection] to the [AssetLoader]
+    ///
+    /// The added collection will be loaded and inserted into your Bevy app as a resource.
+    /// ```edition2021
+    /// # use bevy_asset_loader::{AssetLoader, AssetCollection};
+    /// # use bevy::prelude::*;
+    /// # use bevy::asset::AssetPlugin;
+    /// # fn main() {
+    ///     let mut app = App::build();
+    ///     AssetLoader::new(GameState::Loading)
+    ///         .continue_to_state(GameState::Menu)
     ///         .with_collection::<AudioAssets>()
     ///         .with_collection::<TextureAssets>()
     ///         .build(&mut app);
@@ -356,14 +495,24 @@ where
         self
     }
 
-    /// Add any [FromWorld] resource to be inititlized after all asset collections are loaded.
-    /// ```edition2018
+    /// Insert a map of asset keys with corresponding asset paths
+    pub fn add_keys(mut self, mut keys: HashMap<String, String>) -> Self {
+        keys.drain().for_each(|(key, value)| {
+            self.keys.insert(key, value);
+        });
+
+        self
+    }
+
+    /// Add any [bevy_ecs::world::FromWorld] resource to be initialized after all asset collections are loaded.
+    /// ```edition2021
     /// # use bevy_asset_loader::{AssetLoader, AssetCollection};
     /// # use bevy::prelude::*;
     /// # use bevy::asset::AssetPlugin;
     /// # fn main() {
     ///     let mut app = App::build();
-    ///     AssetLoader::new(GameState::Loading, GameState::Menu)
+    ///     AssetLoader::new(GameState::Loading)
+    ///         .continue_to_state(GameState::Menu)
     ///         .with_collection::<TextureForAtlas>()
     ///         .init_resource::<TextureAtlasFromWorld>()
     ///         .build(&mut app);
@@ -409,13 +558,14 @@ where
     /// Finish configuring the [AssetLoader]
     ///
     /// Calling this function is required to set up the asset loading.
-    /// ```edition2018
+    /// ```edition2021
     /// # use bevy_asset_loader::{AssetLoader, AssetCollection};
     /// # use bevy::prelude::*;
     /// # use bevy::asset::AssetPlugin;
     /// # fn main() {
     ///     let mut app = App::build();
-    ///     AssetLoader::new(GameState::Loading, GameState::Menu)
+    ///     AssetLoader::new(GameState::Loading)
+    ///         .continue_to_state(GameState::Menu)
     ///         .with_collection::<AudioAssets>()
     ///         .with_collection::<TextureAssets>()
     ///         .build(&mut app);
@@ -463,8 +613,14 @@ where
                 .insert(self.loading_state.clone(), config);
             app.world_mut().insert_resource(asset_loader_configuration);
         }
+        app.init_resource::<AssetKeys>();
         app.add_system_set(self.load)
             .add_system_set(self.check)
             .add_system_set(self.post_process);
     }
 }
+
+#[cfg(feature = "render")]
+#[doc = include_str!("../../README.md")]
+#[cfg(doctest)]
+struct ReadmeDoctests;
